<?php
/**
 * Simply Schedule Appointments Appointments Model.
 *
 * @since   0.0.3
 * @package Simply_Schedule_Appointments
 */
use League\Period\Period;

/**
 * Simply Schedule Appointments Appointments Model.
 *
 * @since 0.0.3
 */
class SSA_Appointment_Model extends SSA_Db_Model {
	protected $slug = 'appointment';
	protected $version = '1.3';

	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.2
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.0.2
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		// $this->version = $this->version.'.'.time(); // dev mode
		parent::__construct( $plugin );

		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.0.2
	 */
	public function hooks() {
		// add_action( 'init', array($this,'debug') );
	}

	public function debug() {
	}

	public function belongs_to() {
		return array(
			'Author' => array(
				'model' => 'WP_User_Model',
				'foreign_key' => 'author_id',
			),
			'AppointmentType' => array(
				'model' => $this->plugin->appointment_type_model,
				'foreign_key' => 'appointment_type_id',
			),
		);
	}

	protected $schema = array(
		'appointment_type_id' => array(
			'field' => 'appointment_type_id',
			'label' => 'Appointment Type ID',
			'default_value' => 0,
			'format' => '%d',
			'mysql_type' => 'BIGINT',
			'mysql_length' => 20,
			'mysql_unsigned' => true,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'author_id' => array(
			'field' => 'author_id',
			'label' => 'Author ID',
			'default_value' => 0,
			'format' => '%d',
			'mysql_type' => 'BIGINT',
			'mysql_length' => 20,
			'mysql_unsigned' => true,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'customer_id' => array(
			'field' => 'customer_id',
			'label' => 'Customer ID',
			'default_value' => 0,
			'format' => '%d',
			'mysql_type' => 'BIGINT',
			'mysql_length' => 20,
			'mysql_unsigned' => true,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'customer_information' => array(
			'field' => 'customer_information',
			'label' => 'Customer Information',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'TEXT',
			'mysql_length' => false,
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
			'encoder' => 'json',
		),
		'start_date' => array(
			'field' => 'start_date',
			'label' => 'Start Date',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'datetime',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'end_date' => array(
			'field' => 'end_date',
			'label' => 'End Date',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'datetime',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'title' => array(
			'field' => 'title',
			'label' => 'Title',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'VARCHAR',
			'mysql_length' => '255',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'description' => array(
			'field' => 'description',
			'label' => 'Description',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'TEXT',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'price_expected' => array(
			'field' => 'price_expected',
			'label' => 'Price Expected',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'DECIMAL(9,2)',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'price_full' => array(
			'field' => 'price_full',
			'label' => 'Price Full',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'DECIMAL(9,2)',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'price_discount' => array(
			'field' => 'price_discount',
			'label' => 'Price Discount',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'DECIMAL(9,2)',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'price_final' => array(
			'field' => 'price_final',
			'label' => 'Price Final',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'DECIMAL(9,2)',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'payment_received' => array(
			'field' => 'payment_received',
			'label' => 'Payment Received',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'DECIMAL(9,2)',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'google_calendar_id' => array(
			'field' => 'google_calendar_id',
			'label' => 'Google Calendar ID',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'VARCHAR',
			'mysql_length' => '250',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'google_calendar_event_id' => array(
			'field' => 'google_calendar_event_id',
			'label' => 'Google Calendar Event ID',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'VARCHAR',
			'mysql_length' => '250',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'status' => array(
			'field' => 'status',
			'label' => 'Status',
			'default_value' => 'booked',
			'format' => '%s',
			'mysql_type' => 'VARCHAR',
			'mysql_length' => '16',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'date_created' => array(
			'field' => 'date_created',
			'label' => 'Date Created',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'datetime',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
		'date_modified' => array(
			'field' => 'date_modified',
			'label' => 'Date Modified',
			'default_value' => false,
			'format' => '%s',
			'mysql_type' => 'datetime',
			'mysql_length' => '',
			'mysql_unsigned' => false,
			'mysql_allow_null' => false,
			'mysql_extra' => '',
			'cache_key' => false,
		),
	);

	public $indexes = array(
		'customer_id' => [ 'customer_id' ],
		'start_date' => [ 'start_date' ],
		'end_date' => [ 'end_date' ],
		'status' => [ 'status' ],
		'date_created' => [ 'date_created' ],
	);

	public function filter_where_conditions( $where, $args ) {
		if ( !empty( $args['appointment_type_id'] ) ) {
			$where .= ' AND appointment_type_id="'.sanitize_text_field( $args['appointment_type_id'] ).'"';
		}

		if ( !empty( $args['status'] ) ) {
			$where .= ' AND status="'.sanitize_text_field( $args['status'] ).'"';
		}
		
		return $where;
	}

	public function create_item_permissions_check( $request ) {
		return $this->nonce_permissions_check( $request );
	}

	/**
	 * Check if a given request has access to update a specific item
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|bool
	 */
	public function update_item_permissions_check( $request ) {
		if ( true === $this->get_item_permissions_check( $request ) ) {
			return true;
		}
		
		return false;
	}

	public function create_item( $request ) {
		$params = $request->get_params();
		$params = shortcode_atts( array_merge( $this->get_field_defaults(), array(
			'appointment_type_id' => '',
			'start_date' => '',
			'customer_information' => array(),
			'customer_id' => get_current_user_id(),
			'fetch' => array(),
		) ), $params );

		if ( !empty( $this->schema['customer_id'] ) ) {
			if ( current_user_can( 'manage_options' ) ) {
				// let admins specify the customer_id
			} else {
				$params['customer_id'] = get_current_user_id();
			}
		}

		$request->set_body_params( $params );

		$response = parent::create_item( $request );

		if ( is_a( $response->data['data'], 'WP_Error' ) ) {
			return $response;
		}
		$appointment_object = new SSA_Appointment_Object( $response->data['data']['id'] );
		$response->data['data'] = $appointment_object->get_data( 0, $params['fetch'] );

		// $response->data['data']['ics']['customer'] = $appointment_object->get_ics( 'customer' )['file_url'];

		// if ( current_user_can( 'manage_options' ) ) {
		// 	$response->data['data']['ics']['staff'] = $appointment_object->get_ics( 'staff' )['file_url'];
		// }

		// $response->data['data']['gcal']['customer'] = $appointment_object->get_gcal_add_link( 'customer' );

		return $response;
	}

	public function update_item( $request ) {
		$item_id = $request['id'];
		$params = $request->get_params();
		if ( empty( $params['fetch'] ) ) {
			$params['fetch'] = array();
		}

		$request->set_body_params( $params );
		$response = parent::update_item( $request );
		
		$appointment_object = new SSA_Appointment_Object( $response->data['data']['id'] );
		$response->data['data'] = $appointment_object->get_data( 0, $params['fetch'] );

		if ( is_a( $response->data['data'], 'WP_Error' ) ) {
			return $response;
		}

		return $response;
	}


	public function register_custom_routes() {
		$namespace = $this->api_namespace.'/v' . $this->api_version;
		$base = $this->get_api_base();

		register_rest_route( $namespace, '/' . $base . '/(?P<id>[\d]+)/ics', array(
			array(
				'methods'         => WP_REST_Server::READABLE,
				'callback'        => array( $this, 'get_item_ics' ),
				'permission_callback' => array( $this, 'get_item_permissions_check' ),
				'args'            => array(
					'context'          => array(
						'default'      => 'view',
					),
				),
			),
		) );
	}

	public function get_id_token( $appointment ) {
		if ( empty( $appointment['id'] ) ) {
			return false;
		}

		$appointment_id = sanitize_text_field( $appointment['id'] );
		$string_to_tokenize = $appointment_id;

		if ( empty( $appointment['appointment_type_id'] ) ) {
			$appointment = $this->get( $appointment['id'], -1 );
		}
		if ( !empty( $appointment['id'] ) && !empty( $appointment['appointment_type_id'] ) ) {
			$string_to_tokenize .=  $appointment['appointment_type_id'];
		}
		return SSA_Utils::hash( $string_to_tokenize );
	}

	public function get_item_permissions_check( $request ) {
		$params = $request->get_params();
		if ( true === parent::get_item_permissions_check( $request ) ) {
			return true;
		}

		if ( true === $this->id_token_permissions_check( $request ) ) {
			return true;
		}
		
		if ( is_user_logged_in() ) {
			$appointment = new SSA_Appointment_Object( $params['id'] );
			if ( $appointment->customer_id == get_current_user_id() ) {
				return true;
			}
		}

		return false;
	}

	public function get_item_ics( $request ) {
		$params = $request->get_params();
		$appointment_object = new SSA_Appointment_Object( $params['id'] );
		$ics_exporter = $appointment_object->get_ics_exporter();
		$customer_ics = $ics_exporter->get_ics_for_appointment( $appointment_object, 'customer' );
		$response = array(
			'customer' => $customer_ics,
		);
		
		if ( current_user_can( 'manage_options' ) ) {
			$staff_ics = $ics_exporter->get_ics_for_appointment( $appointment_object, 'staff' );
			$response['staff'] = $staff_ics;
		}

		return $response;
	}

	public function insert( $data, $type = '' ) {
		$response = array();

		$wp_error = new WP_Error();
		if ( empty( $data['appointment_type_id'] ) ) {
			$wp_error->add( 422, 'appointment_type_id required' );
		}
		if ( empty( $data['start_date'] ) ) {
			$wp_error->add( 422, 'start_date required' );
		}
		if ( empty( $data['customer_information'] ) ) {
			$wp_error->add( 422, 'customer_information required' );
		}

		if ( !empty( $wp_error->errors ) ) {
			return $wp_error;
		}
		
		$data['appointment_type_id'] = sanitize_text_field( $data['appointment_type_id'] );
		$data['start_date'] = sanitize_text_field( $data['start_date'] );

		$appointment_type = $this->plugin->appointment_type_model->get( $data['appointment_type_id'] );
		$bookable_period = Period::createFromDuration( $data['start_date'], new DateInterval('PT'.$appointment_type['duration'].'M'));
		$data['end_date'] = $bookable_period->getEndDate()->format( 'Y-m-d H:i:s' );

		$appointment_id = parent::insert( $data, $type );

		do_action( 'ssa/appointment/booked', $appointment_id, $data );

		return $appointment_id;
	}

	public function get_public_edit_url( $appointment_id, $appointment=array() ) {
		$appointment['id'] = $appointment_id;
		$token = $this->get_id_token( $appointment );
		$url = home_url( trailingslashit( $this->plugin->shortcodes->get_appointment_edit_permalink() ) . $token . $appointment_id );
		return $url;
	}

	public function prepare_item_for_response( $item, $recursive=0 ) {
		$item = parent::prepare_item_for_response( $item, $recursive );
		$item['public_edit_url'] = $this->get_public_edit_url( $item['id'], $item );

		return $item;
	}

}
