<?php
/**
 * Simply Schedule Appointments Appointment Object.
 *
 * @since   0.0.3
 * @package Simply_Schedule_Appointments
 */

/**
 * Simply Schedule Appointments Appointment Object.
 *
 * @since 0.0.3
 */
class SSA_Appointment_Object {
	protected $id = null;
	protected $model = null;
	protected $data = null;
	protected $recursive_fetched = -2;

	protected $status;

	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.3
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.0.3
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $id ) {
		$this->plugin = Simply_Schedule_Appointments::get_instance();
		$this->model = $this->plugin->appointment_model;

		if ( $id === 'transient' ) {		
			return;
		}

		$this->id = $id;

		$this->get();
		if ( empty( $this->data['id'] ) || $this->id != $this->data['id'] ) {
			throw new Exception("Unable to create SSA_Appointment_Object from id $id");
		}
	}

	/**
	 * Factory function to create with explicit data
	 *
	 * @param array $data
	 * @return SSA_Appointment_Object
	 * @author 
	 **/
	public static function create( array $data ) {
		$appointment_object = new SSA_Appointment_Object( 'transient' );
		$appointment_object->data = $data;
		return $appointment_object;
	}

	/**
	 * Magic getter for our object.
	 *
	 * @since  0.0.0
	 *
	 * @param  string $field Field to get.
	 * @throws Exception     Throws an exception if the field is invalid.
	 * @return mixed         Value of the field.
	 */
	public function __get( $field ) {
		switch ( $field ) {
			case 'id':
			case 'data':
				return $this->$field;
			case 'start_date_datetime':
			case 'end_date_datetime':
				$date_time = ssa_datetime( $this->data[str_replace('_datetime', '', $field)] );
				return $date_time;
			case 'start_date_timestamp':
			case 'end_date_timestamp':
				return ssa_gmtstrtotime( $this->data[str_replace('_timestamp', '', $field)] );
			default:
				if ( isset( $this->data[$field] ) ) {
					return $this->data[$field];
				}
				
				throw new Exception( 'Invalid ' . __CLASS__ . ' property: ' . $field );
		}
	}

	public function get( $recursive = -1 ) {
		if ( $recursive > $this->recursive_fetched ) {
			if ( null === $this->data ) {
				$this->data = array();
			}
			$this->data = array_merge( $this->data, $this->model->get( $this->id, $recursive ) );
			$this->recursive_fetched = $recursive;
		}
	}

	public function get_data( $recursive = -1, $fetch_fields = array() ) {
		if ( empty( $this->data['id'] ) ) {
			$this->get( $recursive );
		}

		if ( $recursive >= 0 ) {
			if ( !isset( $fetch_fields['public_edit_url'] ) ) {
				$fetch_fields['public_edit_url'] = true;
			}
		}

		if ( !empty( $fetch_fields ) ) {
			$this->fetch_fields( $fetch_fields );
		}

		return $this->data;
	}

	public function fetch_fields( $fetch_fields = array() ) {
		if ( !is_array( $fetch_fields ) ) {
			throw new SSA_Exception("$fetch_fields must be an array", 1);
		}

		foreach ( $fetch_fields as $fetch_field => $fetch_options ) {
			if ( is_int( $fetch_field ) ) {
				$fetch_field = $fetch_options;
				$fetch_options = array();
			}

			$method_name = 'fetch_'.$fetch_field;
			if ( !method_exists( $this, $method_name ) ) {
				throw new SSA_Exception(__CLASS__ . "->" . $method_name . "() not implemented", 1);
			}

			$this->$method_name( $fetch_options );
		}
	}

	public function fetch_add_to_calendar_links( $atts = array() ) {
		if ( !is_array( $atts ) ) {
			$atts = array();
		}

		$atts = shortcode_atts( array(
			'customer' => true,
		), $atts );

		if ( !empty( $atts['customer'] ) ) {
			$this->data['ics']['customer'] = $this->get_ics( 'customer' )['file_url'];
			$this->data['gcal']['customer'] = $this->get_gcal_add_link( 'customer' );
		}
	}

	public function fetch_public_edit_url( $atts = array() ) {
		$this->data['public_edit_url'] = $this->get_public_edit_url();
	}

	public function get_appointment_type() {
		return $appointment_type = new SSA_Appointment_Type_Object( $this->appointment_type_id );
	}

	public function get_date_timezone( $for_type = '', $for_id = '' ) {
		if ( $for_type === 'staff' ) {
			// TODO: Customize for staff ID
			// TODO: Customize for location ID
			// TODO: Customize for customer
			// TODO: Customize for admin
		} else {
			$settings = $this->plugin->settings->get();
			$date_timezone = new DateTimeZone( $settings['global']['timezone_string'] );
		}

		return $date_timezone;
	}

	public function get_customer_calendar_title() {
		return $this->get_appointment_type()->title .' (' . ucfirst( get_bloginfo( 'name' ) ) . ')';
	}

	public function get_description( $template, $eol = "\r\n" ) {
		$description = '';
		if ( $template == 'staff' ) {
			$customer_information = $this->customer_information;
			foreach ($customer_information as $label => $value) {
				$description .= ucwords( str_replace( '_', ' ', $label ) ) . ': ';
				$description .= $value . $eol;
			}
		} elseif ( $template == 'customer' ) {
			$description = $this->get_appointment_type()->description;
			$instructions = $this->get_appointment_type()->instructions; 
			if ( !empty( $instructions ) ) {
				$description = $instructions . $eol . $description;
			}

			$description .= $eol . "Cancel/Reschedule link:"."\r\n";
			$description .= $this->get_public_edit_url()."\r\n";
		}

		return $description;
	}

	public function get_ics_exporter( $template = 'customer' ) {
		$ics_exporter = new SSA_Ics_Exporter();
		$ics_exporter->template = $template;

		return $ics_exporter;
	}

	public function get_ics( $template = 'customer' ) {
		$ics_exporter = $this->get_ics_exporter( $template );
		$ics = $ics_exporter->get_ics_for_appointment( $this );

		return $ics;
	}

	public function get_gcal_add_link( $template = 'customer' ) {
		$link = $this->plugin->gcal_exporter->get_add_link_from_appointment( $this, $template );

		return $link;
	}


	public function is_all_day() {
		return false;
	}

	public function get_public_edit_url() {
		$url = $this->plugin->appointment_model->get_public_edit_url( $this->id );
		return $url;
	}

}
