<?php
/**
 * Simply Schedule Appointments Blackout Dates.
 *
 * @since   0.1.0
 * @package Simply_Schedule_Appointments
 */
use League\Period\Period;

/**
 * Simply Schedule Appointments Blackout Dates.
 *
 * @since 0.1.0
 */
class SSA_Blackout_Dates {
	/**
	 * Parent plugin class.
	 *
	 * @since 0.1.0
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.1.0
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		$this->plugin = $plugin;
		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.1.0
	 */
	public function hooks() {
		add_filter( 'ssa/get_blocked_periods/blocked_periods', array( $this, 'filter_blocked_periods' ), 5, 3 );
	}

	public function filter_blocked_periods( $blocked_periods, $appointment_type, $args ) {
		if ( !$this->plugin->settings_installed->is_enabled( 'blackout_dates' ) ) {
			return $blocked_periods;
		}
		
		$array_of_blocked_dates = $this->plugin->blackout_dates_settings->get();
		if ( empty( $array_of_blocked_dates['dates'] ) || !is_array( $array_of_blocked_dates['dates'] ) ) {
			return $blocked_periods;
		}

		foreach ($array_of_blocked_dates['dates'] as $key => $blocked_date_string ) {
			$blocked_start_local = $this->plugin->utils->get_datetime_as_local_datetime( $blocked_date_string );
			$blocked_period = new Period( $blocked_start_local, $blocked_start_local->add( new DateInterval( 'P1D' ) ) );
			$blocked_periods[] = $blocked_period;
		}

		return $blocked_periods;
	}
}
