<?php
/**
 * Simply Schedule Appointments Notifications.
 *
 * @since   0.0.3
 * @package Simply_Schedule_Appointments
 */

/**
 * Simply Schedule Appointments Notifications.
 *
 * @since 0.0.3
 */
class SSA_Notifications {
	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.3
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.0.3
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		$this->plugin = $plugin;
		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.0.3
	 */
	public function hooks() {
		add_action( 'ssa/appointment/after_insert', array( $this, 'booked_staff_notification' ), 10, 1 );
		add_action( 'ssa/appointment/after_insert', array( $this, 'booked_customer_notification' ), 10, 1 );
		add_action( 'ssa/appointment/after_update', array( $this, 'canceled_staff_notification' ), 10, 4 );
		add_action( 'ssa/appointment/after_update', array( $this, 'canceled_customer_notification' ), 10, 4 );
	}

	public function get_template( $template_name ) {

	}

	private function render_template_content( $template_content, $data ) {
		
	}

	private function render_template( $template, $template_variation, $data ) {
		$appointment = new SSA_Appointment_Object( $data['id'] );

		$content = '';
		if ( $template == 'booked' && $template_variation == 'staff' ) {
			$content .= $data['customer_information']['Name'].' just booked an appointment on '.home_url()."\r\n";
			$content .= "\r\n";
			$content .= 'Appointment Details:'."\r\n";
			$content .= 'Starting at '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y g:ia (T)')."\r\n";
			if ( $instructions = $appointment->get_appointment_type()->instructions ) {
				$content .= 'Instructions: '.$instructions."\r\n";
			}
			$content .= 'Type: '.$appointment->get_appointment_type()->title."\r\n";
			$content .= "\r\n";
			foreach ($data['customer_information'] as $key => $value) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', array_values( $value ) );
				}

				if ( empty( trim( $value ) ) ) {
					continue;
				}

				$content .= ucwords( str_replace( '_', ' ', $key ) ) . ': ' . $value . "\r\n";
			}
		} elseif ( $template == 'booked' && $template_variation == 'customer' ) {
			$content .= 'Hi '.$data['customer_information']['Name'].','."\r\n"."\r\n";
			$content .= 'This is a confirmation of the appointment you just booked on '.home_url()."\r\n";
			$content .= "\r\n";
			$content .= 'Appointment scheduled for '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y g:ia (T)')."\r\n";
			if ( $instructions = $appointment->get_appointment_type()->instructions ) {
				$content .= 'Instructions: '.$instructions."\r\n";
			}
			$content .= "\r\n";
			foreach ($data['customer_information'] as $key => $value) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', array_values( $value ) );
				}

				if ( empty( trim( $value ) ) ) {
					continue;
				}
				
				$content .= ucwords( str_replace( '_', ' ', $key ) ) . ': ' . $value . "\r\n";
			}
			$content .= "\r\n";
			$content .= "If you need to cancel or change your appointment, you can do so by visiting this link:"."\r\n";
			$content .= $appointment->get_public_edit_url()."\r\n";
		} elseif ( $template == 'canceled' && $template_variation == 'staff' ) {
			$content .= 'Your appointment "'.$appointment->get_appointment_type()->title.'"';
			$content .= ' with '.$appointment->customer_information['Name'];
			$content .= ' on '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y');
			$content .= ' has been canceled';
			$content .= "\r\n"."\r\n"."\r\n";
			$content .= 'Customer Details:'."\r\n"."\r\n";
			foreach ($data['customer_information'] as $key => $value) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', array_values( $value ) );
				}

				if ( empty( trim( $value ) ) ) {
					continue;
				}

				$content .= ucwords( str_replace( '_', ' ', $key ) ) . ': ' . $value . "\r\n";
			}
		} elseif ( $template == 'canceled' && $template_variation == 'customer' ) {
			$content .= 'Your '.$appointment->get_appointment_type()->title;
			$content .= ' on '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y');
			$content .= ' has been canceled';
		}

		$content = apply_filters( 'ssa/notifications/render_template', $content, $template, $template_variation, $data );

		return $content;
	}

	/**
	 * Get mail headers (From/Cc/Bcc) for a given appointment or appointment type
	 *
	 * @param SSA_Appointment_Object $appointment_object
	 * @param string $template
	 * @return array
	 * @author 
	 **/
	public function get_mail_headers_for_appointment( SSA_Appointment_Object $appointment_object, $template ) {
		$headers = array();

		
	}

	public function get_from_name_for_appointment( SSA_Appointment_Object $appointment_object, $recipient, $template ) {
		$settings = $this->plugin->settings->get();

		if ( $recipient == 'customer' ) {
			$value = str_replace( '"', '', $settings['global']['staff_name'] ) .' at '.str_replace( '"', '', $settings['global']['company_name'] );
		} elseif ( $recipient == 'staff' ) {
			$value = $appointment_object->customer_information['Name'] .' ('.$settings['global']['company_name'].')';
		}

		return $value;
	}

	// public function get_from_email_for_appointment( SSA_Appointment_Object $appointment_object, $template ) {
	// 	$settings = $this->plugin->settings->get();

	// 	$value = str_replace( '"', '', $settings['global']['staff_name'] ) .' at '.str_replace( '"', '', $settings['global']['company_name'] );

	// 	return $value;
	// }

	public function get_reply_to_email_for_appointment( SSA_Appointment_Object $appointment_object, $recipient, $template ) {
		$settings = $this->plugin->settings->get();

		if ( $recipient == 'customer' ) {
			$value = $settings['global']['admin_email'];
		} elseif ( $recipient == 'staff' ) {
			$value = $appointment_object->customer_information['Email'];
		}
		
		return $value;
	}

	public function set_ssa_from_name( $name ) {
		global $ssa_wp_mail_from_name;
		global $ssa_wp_mail_from_name_swp;
		$ssa_wp_mail_from_name_swp = $ssa_wp_mail_from_name;
		$ssa_wp_mail_from_name = $name;
	}

	public function reset_ssa_from_name() {
		global $ssa_wp_mail_from_name;
		global $ssa_wp_mail_from_name_swp;
		$ssa_wp_mail_from_name = $ssa_wp_mail_from_name_swp;
		unset( $ssa_wp_mail_from_name_swp );
	}

	public function get_ssa_from_name() {
		global $ssa_wp_mail_from_name;
		return $ssa_wp_mail_from_name;
	}

	public function wp_mail( $to, $subject, $message, $headers = '', $attachments = array(), $from_name = '', $from_email = '' ) {
		$this->set_ssa_from_name( $from_name );

		add_filter( 'wp_mail_from_name', array( $this, 'get_ssa_from_name' ) );
		wp_mail( $to, $subject, $message, $headers, $attachments );
		remove_filter( 'wp_mail_from_name', array( $this, 'get_ssa_from_name' ) );

		$this->reset_ssa_from_name();
	}

	public function canceled_staff_notification( $data_after, $data_before, $appointment_id, $response ) {
		if ( empty( $data_after['status'] ) || empty( $data_before['status'] ) ) {
			return;
		}
		if ( $data_after['status'] !== 'canceled' || $data_before['status'] !== 'booked' ) {
			return; // we only want to send a notification when an appointment goes from booked -> canceled
		}

		$appointment = new SSA_Appointment_Object( $appointment_id );
		$settings = $this->plugin->settings->get();
		$notification_settings = $appointment->get_appointment_type()->notifications;
		foreach ($notification_settings as $notification_settings_key => $notification_settings ) {
			if ( $notification_settings['field'] == 'admin' && !empty( $notification_settings['send'] ) ) {		
				$this->wp_mail(
					$settings['global']['admin_email'],
					'Canceled: '.$appointment->get_appointment_type()->title.' with '.$appointment->customer_information['Name'].' on '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y'),
					$this->render_template( 'canceled', 'staff', $appointment->data ),
					array(
						'Reply-To: '.$this->get_reply_to_email_for_appointment( $appointment, 'staff', 'canceled' ),
						'Content-Type: text/plain',
					),
					array(),
					$this->get_from_name_for_appointment( $appointment, 'staff', 'canceled' )
				);
			}
		}

	}

	public function canceled_customer_notification( $data_after, $data_before, $appointment_id, $response ) {
		if ( empty( $data_after['status'] ) || empty( $data_before['status'] ) ) {
			return;
		}

		if ( $data_after['status'] !== 'canceled' || $data_before['status'] !== 'booked' ) {
			return; // we only want to send a notification when an appointment goes from booked -> canceled
		}

		$appointment = new SSA_Appointment_Object( $appointment_id );
		if ( empty( $appointment->customer_information['Email'] ) ) {
			return;
		}

		$settings = $this->plugin->settings->get();
		$notification_settings = $appointment->get_appointment_type()->notifications;
		foreach ($notification_settings as $notification_settings_key => $notification_settings ) {
			if ( $notification_settings['field'] == 'customer' && !empty( $notification_settings['send'] ) ) {		
				$this->wp_mail(
					$appointment->customer_information['Email'],
					'Canceled: '.$appointment->get_appointment_type()->title.' on '.$appointment->start_date_datetime->setTimezone( $appointment->get_date_timezone() )->format('F d, Y'),
					$this->render_template( 'canceled', 'customer', $appointment->data ),
					array(
						'Reply-To: '.$this->get_reply_to_email_for_appointment( $appointment, 'customer', 'canceled' ),
						'Content-Type: text/plain',
					),
					array(),
					$this->get_from_name_for_appointment( $appointment, 'customer', 'canceled' )
				);
			}
		}


	}
	public function booked_staff_notification( $appointment_id ) {
		$appointment = new SSA_Appointment_Object( $appointment_id );
		$settings = $this->plugin->settings->get();
		$notification_settings = $appointment->get_appointment_type()->notifications;
		foreach ($notification_settings as $notification_settings_key => $notification_settings ) {
			if ( $notification_settings['field'] == 'admin' && !empty( $notification_settings['send'] ) ) {
				$this->wp_mail(
					$settings['global']['admin_email'],
					$appointment->customer_information['Name']. ' just booked an appointment - '.$appointment->get_appointment_type()->title,
					$this->render_template( 'booked', 'staff', $appointment->data ),
					array(
						'Reply-To: '.$this->get_reply_to_email_for_appointment( $appointment, 'staff', 'booked' ),
						'Content-Type: text/plain',
					),
					array(),
					$this->get_from_name_for_appointment( $appointment, 'staff', 'booked' )
				);				
			}
		}


	}

	public function booked_customer_notification( $appointment_id ) {
		$appointment = new SSA_Appointment_Object( $appointment_id );
		if ( empty( $appointment->customer_information['Email'] ) ) {
			return;
		}
		if ( $appointment->status !== 'booked' ) {
			return;
		}

		$settings = $this->plugin->settings->get();
		$notification_settings = $appointment->get_appointment_type()->notifications;
		foreach ($notification_settings as $notification_settings_key => $notification_settings ) {
			if ( $notification_settings['field'] == 'customer' && !empty( $notification_settings['send'] ) ) {		
				$this->wp_mail(
					$appointment->customer_information['Email'],
					'Your appointment details',
					$this->render_template( 'booked', 'customer', $appointment->data ),
					array(
						'Reply-To: '.$this->get_reply_to_email_for_appointment( $appointment, 'customer', 'booked' ),
						'Content-Type: text/plain',
					),
					array(),
					$this->get_from_name_for_appointment( $appointment, 'customer', 'booked' )
				);
			}
		}

	}
}