<?php
/**
 * Simply Schedule Appointments Settings Installed.
 *
 * @since   0.1.0
 * @package Simply_Schedule_Appointments
 */

/**
 * Simply Schedule Appointments Settings Installed.
 *
 * @since 0.1.0
 */
class SSA_Settings_Installed extends SSA_Settings_Schema {
	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.3
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.0.3
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		parent::__construct();
		$this->plugin = $plugin;
		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.0.3
	 */
	public function hooks() {
	}

	protected $slug = 'installed';

	public function get_schema() {
		if ( !empty( $this->schema ) ) {
			return $this->schema;
		}

		$this->schema = array(
			'version' => '2018-03-15',
			'fields' => array(
				
			),
		);

		return $this->schema;
	}

	public function get_computed_schema() {
		if ( !empty( $this->computed_schema ) ) {
			return $this->computed_schema;
		}

		$this->computed_schema = array(
			'fields' => array(
				'blackout_dates' => array(
					'name' => 'blackout_dates',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Blackout_Dates',
				),

				'advanced_scheduling' => array(
					'name' => 'advanced_scheduling',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Advanced_Scheduling_Settings',
				),

				'google_calendar' => array(
					'name' => 'google_calendar',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Google_Calendar',
				),

				'license' => array(
					'name' => 'license',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_License',
				),

				'mailchimp' => array(
					'name' => 'mailchimp',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Mailchimp',
				),

				'styles' => array(
					'name' => 'styles',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Styles',
				),

				'webhooks' => array(
					'name' => 'webhooks',
					'get_function' => 'class_exists',
					'get_input' => 'SSA_Webhooks',
				),
			),
		);

		return $this->computed_schema;
	}

	public function is_installed( $slug ) {
		$installed = $this->plugin->settings_installed->get();
		if ( !empty( $installed[$slug] ) ) {
			return true;
		}

		return false;
	}

	public function is_enabled( $slug ) {
		if ( ! $this->is_installed( $slug ) ) {
			return false;
		}

		$settings = $this->plugin->settings->get();
		if ( !empty( $settings[$slug]['enabled'] ) ) {
			return true;
		}

		return false;
	}
}
