<?php
/**
 * Simply Schedule Appointments Shortcodes.
 *
 * @since   0.0.3
 * @package Simply_Schedule_Appointments
 */

/**
 * Simply Schedule Appointments Shortcodes.
 *
 * @since 0.0.3
 */
class SSA_Shortcodes {
	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.3
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;
	protected $script_handle_whitelist = array();
	protected $style_handle_whitelist = array();
	protected $custom_script_handle_whitelist = array();
	protected $disable_third_party_styles = false;
	protected $disable_third_party_scripts = false;

	/**
	 * Constructor.
	 *
	 * @since  0.0.3
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		$this->plugin = $plugin;
		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.0.3
	 */
	public function hooks() {
		add_shortcode( 'ssa_booking', array( $this, 'ssa_booking' ) );

		add_action( 'init', array( $this, 'store_enqueued_styles_scripts' ), 0 );

		add_action( 'wp_enqueue_scripts', array( $this, 'register_styles' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'register_scripts' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );

		add_action( 'wp_enqueue_scripts', array( $this, 'disable_third_party_scripts' ), 9999999 );
		add_action( 'wp_enqueue_scripts', array( $this, 'disable_third_party_styles' ), 9999999 );
		add_filter( 'template_include', array( $this, 'hijack_booking_page_template' ) );
		add_filter( 'template_include', array( $this, 'hijack_embedded_page' ), 9999999 );
		add_filter( 'template_include', array( $this, 'hijack_appointment_edit_page' ), 9999999 );
	}

	public function store_enqueued_styles_scripts() {
		if ( is_admin() ) {
			return;
		}

		global $wp_scripts;
		$this->script_handle_whitelist = $wp_scripts->queue;
	}

	public function disable_third_party_scripts() {
		if ( !$this->disable_third_party_scripts ) {
			return;
		}

		global $wp_scripts;
		foreach ($wp_scripts->queue as $key => $handle) {
			if ( strpos( $handle, 'ssa-' ) === 0 ) {
				continue;
			}

			if ( in_array( $handle, $this->script_handle_whitelist ) || in_array( $handle, $this->custom_script_handle_whitelist ) ) {
				continue;
			}

			wp_dequeue_script( $handle );
		}
	}

	public function disable_third_party_styles() {
		if ( !$this->disable_third_party_styles ) {
			return;
		}

		global $wp_styles;
		foreach ($wp_styles->queue as $key => $handle) {
			if ( strpos( $handle, 'ssa-' ) === 0 ) {
				continue;
			}

			if ( in_array( $handle, $this->style_handle_whitelist ) || in_array( $handle, $this->custom_script_handle_whitelist ) ) {
				continue;
			}

			wp_dequeue_style( $handle );
		}
	}

	public function is_booking_page() {
		$page_id = get_queried_object_id();
		$settings = $this->plugin->settings->get();
		if ( empty( $page_id ) || $page_id != $settings['global']['booking_post_id'] ) {
			return false;
		}

		return true;
	}

	public function get_appointment_edit_permalink() {
		return apply_filters( 'ssa/booking/appointment_edit_permalink', 'appointments/change' );
	}

	public function is_embedded_page() {
		if ( !empty( $_GET['ssa_embed'] ) ) {
			return true;
		}

		global $wp;
		if ( $wp->request === 'ssa-embed' ) {
			return true;
		}

		return false;
	}

	public function hijack_embedded_page( $template ) {
		if ( ! $this->is_embedded_page() ) {
			return $template;
		}

		return $this->plugin->dir( 'booking-app/iframe-inner.php' );
	}

	public function hijack_booking_page_template( $template ) {
		if ( !$this->is_booking_page() ) {
			return $template;
		}

		return $this->plugin->dir( 'booking-app/fullscreen-page.php' );
	}

	public function hijack_appointment_edit_page( $template ) {
		$appointment_edit_permalink = $this->get_appointment_edit_permalink();
		global $wp;
		$uri = sanitize_text_field( $wp->request );
		if ( strpos( $uri, $appointment_edit_permalink ) !== 0 ) {
			return $template;
		}

		$uri = str_replace( $appointment_edit_permalink, '', $uri );
		$uri = ltrim( $uri, '/' );
		$uri = rtrim( $uri, '/' );
		$parts = explode( '-', $uri );

		$provided_hash = $parts['0'];
		$provided_hash = substr( $uri, 0, 32 );
		$appointment_id = substr( $uri, 32 );
		$correct_hash = $this->plugin->appointment_model->get_id_token( array( 'id' => $appointment_id ) );

		if ( $correct_hash != $provided_hash ) {;
			die( 'An error occurred, please check the URL' );
		}
		
		global $ssa_current_appointment_id;
		$ssa_current_appointment_id = $appointment_id;
		return $this->plugin->dir( 'booking-app/page-appointment-edit.php' );
	}

	public function body_class( $classes ) {
		$classes = "$classes ssa-admin-app";

		return $classes;
	}


	public function register_scripts() {
		wp_register_script( 'ssa-booking-manifest', $this->plugin->url('booking-app/dist/static/js/manifest.js'), array(), Simply_Schedule_Appointments::VERSION, true );
		wp_register_script( 'ssa-booking-vendor', $this->plugin->url('booking-app/dist/static/js/vendor.js'), array( 'ssa-booking-manifest' ), Simply_Schedule_Appointments::VERSION, true );
		wp_register_script( 'ssa-booking-app', $this->plugin->url('booking-app/dist/static/js/app.js'), array( 'ssa-booking-vendor' ), Simply_Schedule_Appointments::VERSION, true );
		$this->custom_script_handle_whitelist[] = 'ssa-booking-manifest';
		$this->custom_script_handle_whitelist[] = 'ssa-booking-vendor';
		$this->custom_script_handle_whitelist[] = 'ssa-booking-app';

		wp_register_script( 'ssa-iframe-inner', $this->plugin->url('assets/js/iframe-inner.js'), array(), Simply_Schedule_Appointments::VERSION, true );
		wp_register_script( 'ssa-iframe-outer', $this->plugin->url('assets/js/iframe-outer.js'), array(), Simply_Schedule_Appointments::VERSION, true );
	}

	public function register_styles() {
		wp_register_style( 'ssa-booking-material-icons', '//fonts.googleapis.com/icon?family=Material+Icons', array(), Simply_Schedule_Appointments::VERSION );
		wp_register_style( 'ssa-booking-style', $this->plugin->url('booking-app/dist/static/css/app.css'), array(), Simply_Schedule_Appointments::VERSION );
		wp_register_style( 'ssa-booking-roboto-font', '//fonts.googleapis.com/css?family=Roboto:300,400,500,700,400italic', array(), Simply_Schedule_Appointments::VERSION );
		wp_register_style( 'ssa-iframe-inner', $this->plugin->url('assets/css/iframe-inner.css'), array(), Simply_Schedule_Appointments::VERSION );
		wp_register_style( 'ssa-iframe-outer', $this->plugin->url('assets/css/iframe-outer.css'), array(), Simply_Schedule_Appointments::VERSION );
	}

	public function enqueue_styles() {
		global $post;
		if ( $this->is_embedded_page() ) {
			wp_enqueue_style( 'ssa-booking-material-icons' );
			wp_enqueue_style( 'ssa-booking-style' );
			wp_enqueue_style( 'ssa-booking-roboto-font' );
			wp_enqueue_style( 'ssa-iframe-inner' );
		} elseif ( is_a( $post, 'WP_Post' ) && has_shortcode( $post->post_content, 'ssa_booking') ) {
      wp_enqueue_style( 'ssa-iframe-outer');
    }
	}

	public function ssa_booking( $atts ) {
		$atts = shortcode_atts( array(
			'type' => '',
			'edit' => '',
		), $atts, 'ssa_booking' );

		$appointment_type = '';
		if ( !empty( $atts['type'] ) ) {
			if ( $atts['type'] == (string)(int) $atts['type'] ) {
				// integer ID provided
				$appointment_type_id = (int) sanitize_text_field( $atts['type'] );
			} else {
				// slug provided
				$appointment_types = $this->plugin->appointment_type_model->query( array(
					'slug' => sanitize_text_field( $atts['type'] ),
					'status' => 'publish',
				) );
				if ( !empty( $appointment_types['0']['id'] ) ) {
					$appointment_type_id = $appointment_types['0']['id'];
				}

				if ( empty( $appointment_type_id ) ) {
					$error_message = '<h3>Sorry this appointment type isn\'t available, please check back later</h3>';
					if ( current_user_can( 'manage_options' ) ) {
						$error_message .= '<code>The specified appointment type "'.$atts['type'].'" can\'t be found</code> (this message only viewable to site administrators)';
					}

					return $error_message;
				}
			}

			if ( !empty( $appointment_type_id ) ) {
				$appointment_type = $this->plugin->appointment_type_model->get( $appointment_type_id );
			}
		}

		if ( $this->is_embedded_page() ) {
			// wp_localize_script( 'ssa-booking-app', 'ssaBookingParams', array(
			// 	'apptType' => $appointment_type,
			// 	'translatedStrings' => array(
			// 	),
			// ) );
			wp_localize_script( 'ssa-booking-app', 'ssa', array(
				'api' => $this->plugin->bootstrap->get_api_vars(),
			) );
			wp_localize_script( 'ssa-booking-app', 'ssa_translations', $this->get_translations() );

			wp_enqueue_script( 'ssa-booking-manifest' );
			wp_enqueue_script( 'ssa-booking-vendor' );
			wp_enqueue_script( 'ssa-booking-app' );
			wp_enqueue_script( 'ssa-iframe-inner' );

			return '
			<div id="ssa-booking-app"></div>
			';
		}

		wp_enqueue_script( 'ssa-iframe-outer' );
		$settings = $this->plugin->settings->get();
		// $link = get_page_link( $settings['global']['booking_post_id'] );
		$link = add_query_arg( $atts, home_url( 'ssa-embed' ) );
		if ( !empty( $atts['edit'] ) ) {
			$appointment_id = sanitize_text_field( $atts['edit'] );
			$appointment_id_token = $this->plugin->appointment_model->get_id_token( array( 'id' => sanitize_text_field( $atts['edit'] ) ) );
			$link = $link . '#/change/'. $appointment_id_token . $appointment_id;
		} elseif ( ! empty( $appointment_type ) ) {
			$link = $link . '#/type/' . $appointment_type['slug'];
		}

		return '<iframe src="' . $link . '" height="100px" width="100%" name="ssa_booking" frameborder="0" class="ssa_booking_iframe"></iframe>';

	}


	public function get_translations() {
		include $this->plugin->dir( 'languages/booking-app-translations.php' );
		return $translations;
	}

}
