<?php
/**
 * Simply Schedule Appointments Wp Admin.
 *
 * @since   0.0.3
 * @package Simply_Schedule_Appointments
 */

/**
 * Simply Schedule Appointments Wp Admin.
 *
 * @since 0.0.3
 */
class SSA_Wp_Admin {
	protected $script_handle_whitelist = array();
	protected $style_handle_whitelist = array();

	/**
	 * Parent plugin class.
	 *
	 * @since 0.0.3
	 *
	 * @var   Simply_Schedule_Appointments
	 */
	protected $plugin = null;

	/**
	 * Constructor.
	 *
	 * @since  0.0.3
	 *
	 * @param  Simply_Schedule_Appointments $plugin Main plugin object.
	 */
	public function __construct( $plugin ) {
		$this->plugin = $plugin;
		$this->hooks();
	}

	/**
	 * Initiate our hooks.
	 *
	 * @since  0.0.3
	 */
	public function hooks() {
		add_action( 'admin_menu', array( $this, 'register_admin_menu' ) );

		add_action( 'admin_init', array( $this, 'maybe_redirect' ), 0 );
		add_action( 'admin_init', array( $this, 'store_enqueued_styles_scripts' ), 0 );
		add_action( 'admin_enqueue_scripts', array( $this, 'disable_third_party_styles_scripts' ), 9999999 );
		add_action( 'admin_body_class', array( $this, 'body_class' ) );

		add_action('admin_print_scripts', array( $this, 'remove_admin_notices') );

	}

	public function maybe_redirect() {
		if ( !empty( $_GET['page'] ) && $_GET['page'] === 'simply-schedule-appointments-settings' ) {
			wp_redirect( $this->url( '/ssa/settings' ) );
			exit;
		}

		if ( !empty( $_GET['page'] ) && $_GET['page'] === 'simply-schedule-appointments-types' ) {
			wp_redirect( $this->url( '/ssa/appointment-types/all' ) );
			exit;
		}
	}

	public function url( $path='' ) {
		$url = admin_url( 'admin.php?page=simply-schedule-appointments' );
		if ( empty( $path ) ) {
			return $url;
		}

		$path = ltrim( $path, '/' );
		$url .= '/#/'.$path;

		return $url;
	}

	public function remove_admin_notices() {
		if ( !$this->is_admin_page() ) {
			return;
		}
		global $wp_filter;
		if (is_user_admin()) {
			if (isset($wp_filter['user_admin_notices'])) {
				unset($wp_filter['user_admin_notices']);
			}
		} elseif (isset($wp_filter['admin_notices'])) {
			unset($wp_filter['admin_notices']);
		}
		if (isset($wp_filter['all_admin_notices'])) {
			unset($wp_filter['all_admin_notices']);
		}
	}

	public function is_admin_page() {
		if ( empty( $_GET['page'] ) || strpos( $_GET['page'], 'simply-schedule-appointments' ) === false ) {
			return false;
		}

		return true;
	}

	public function store_enqueued_styles_scripts() {
		if ( !$this->is_admin_page() ) {
			return;
		}

		global $wp_scripts;
		$this->script_handle_whitelist = $wp_scripts->queue;
	}

	public function disable_third_party_styles_scripts() {
		if ( !$this->is_admin_page() ) {
			return;
		}

		$custom_whitelist = array(

		);

		global $wp_scripts;
		foreach ($wp_scripts->queue as $key => $handle) {
			if ( strpos( $handle, 'ssa-' ) === 0 ) {
				continue;
			}

			if ( in_array( $handle, $this->script_handle_whitelist ) || in_array( $handle, $custom_whitelist ) ) {
				continue;
			}

			wp_dequeue_script( $handle );
		}

		global $wp_styles;
		foreach ($wp_styles->queue as $key => $handle) {
			if ( strpos( $handle, 'ssa-' ) === 0 ) {
				continue;
			}

			if ( in_array( $handle, $this->style_handle_whitelist ) || in_array( $handle, $custom_whitelist ) ) {
				continue;
			}

			wp_dequeue_style( $handle );
		}

	}

	public function register_admin_menu() {
		add_menu_page( 
			__('Appointments', 'simply-schedule-appointments' ),
			__('Appointments', 'simply-schedule-appointments' ),
			'manage_options',
			'simply-schedule-appointments',
			array( $this, 'render_admin_page' ),
			'dashicons-calendar',
			null
		);

		$settings = $this->plugin->settings->get();
		if ( !empty( $settings['global']['wizard_completed'] ) ) {
			add_submenu_page(
				'simply-schedule-appointments',
				__('Appointment Types', 'simply-schedule-appointments' ),
				__('Appointment Types', 'simply-schedule-appointments' ),
				'manage_options',
				'simply-schedule-appointments-types',
				array( $this, 'render_admin_page' )
			);

			add_submenu_page(
				'simply-schedule-appointments',
				__('Settings', 'simply-schedule-appointments' ),
				__('Settings', 'simply-schedule-appointments' ),
				'manage_options',
				'simply-schedule-appointments-settings',
				array( $this, 'render_admin_page' )
			);	
		}

	}

	public function render_admin_page() {
		wp_enqueue_style( 'ssa-admin-material-icons', '//fonts.googleapis.com/icon?family=Material+Icons', array(), Simply_Schedule_Appointments::VERSION );
		wp_enqueue_style( 'ssa-admin-style', $this->plugin->url('admin-app/dist/static/css/app.css'), array(), Simply_Schedule_Appointments::VERSION );
		wp_enqueue_style( 'ssa-admin-roboto-font', '//fonts.googleapis.com/css?family=Roboto:300,400,500,700,400italic', array(), Simply_Schedule_Appointments::VERSION );

		wp_enqueue_script( 'ssa-admin-manifest', $this->plugin->url('admin-app/dist/static/js/manifest.js'), array(), Simply_Schedule_Appointments::VERSION, true );
		wp_enqueue_script( 'ssa-admin-vendor', $this->plugin->url('admin-app/dist/static/js/vendor.js'), array( 'ssa-admin-manifest' ), Simply_Schedule_Appointments::VERSION, true );
		wp_register_script( 'ssa-admin-app', $this->plugin->url('admin-app/dist/static/js/app.js'), array( 'ssa-admin-vendor' ), Simply_Schedule_Appointments::VERSION, true );
		$dismissed_notices = $this->plugin->notices->get_dismissed_notices();
		if ( count( $dismissed_notices ) ) {
			$dismissed_notices = array_combine( $dismissed_notices, array_fill(0, count( $dismissed_notices ), true ) );
		}
		if ( empty( $dismissed_notices ) ) {
			$dismissed_notices = new stdClass();
		}
		$settings = $this->plugin->settings->get();
		$settings = array_intersect_key( $settings, array(
			'global' => '',
			'installed' => '',
		) );
		wp_localize_script( 'ssa-admin-app', 'ssa_dismissed_notices', $dismissed_notices );
		wp_localize_script( 'ssa-admin-app', 'ssa_settings', $settings );
		wp_localize_script( 'ssa-admin-app', 'ssa_translations', $this->get_translations() );
		wp_enqueue_script( 'ssa-admin-app' );

		echo '
		<style>
			.ssa-admin-app #wpadminbar,
			.ssa-admin-app #adminmenumain,
			.ssa-admin-app #wpfooter,
			.ssa-admin-app .hidden {
				display: none;
			}
		</style>
		<div id="ssa-admin-app"></div>
		';
	}

	public function get_translations() {
		include $this->plugin->dir( 'languages/admin-app-translations.php' );
		return $translations;
	}

	public function body_class( $classes ) {
		if ( !$this->is_admin_page() ) {
			return $classes;
		}

		$classes = "$classes ssa-admin-app";

		return $classes;
	}

	public function maybe_create_booking_page() {
		$settings = $this->plugin->settings->get();
		if ( empty( $settings['global']['booking_post_id'] ) ) {
			return $this->create_booking_page();
		}

		$appointment = $this->plugin->appointment_model->get( $settings['global']['booking_post_id'] );
		if ( empty( $appointment['id'] ) ) {
			return $this->create_booking_page();
		}

		return $settings['global']['booking_post_id'];
	}

	public function create_booking_page() {
		$post_id = wp_insert_post( array(
			'post_type' => 'page',
			'post_status' => 'publish',
			'post_title' => 'Schedule an Appointment',
			'post_name' => 'appointments',
			'post_content' => '[ssa_booking]',
		), $wp_error );

		$settings = $this->plugin->settings->get();
		$settings['global']['booking_post_id'] = $post_id;
		$this->plugin->settings->update_section( 'global', $settings['global'] );

		return $post_id;
	}

}
